package cn.com.qmth.export;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.alibaba.fastjson.JSONObject;

public class ExportProperty {
	private static Logger logger = LogManager.getLogger(ExportProperty.class);
	private static String dbName="yunkai_question5";
	private final static String sourceDir = "d:/yunkai/";
	public static void main(String[] args) {
		logger.debug("导出开始");
		try {
			dispose();
		} catch (Exception e) {
			logger.error(e.getCause(), e);
		}
		logger.debug("导出结束");
	}
	
	private static void dispose() {
		File dir=new File(sourceDir+"prop/");
		if(dir.exists()) {
			FileUtil.clearDirectory(sourceDir+"prop/");
		}else {
			dir.mkdirs();
		}
		Connection connect = null;

		try {
			Class.forName("com.mysql.cj.jdbc.Driver");

			String url = "jdbc:mysql://localhost:3306/"+dbName+"?serverTimezone=GMT%2B8";

			String user = "root";

			String password = "123456";
			connect = DriverManager.getConnection(url, user, password);
			exportProperty(connect);
		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			if (connect != null) {
				try {
					connect.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	private static void exportProperty(Connection connect) throws InvalidFormatException, IOException, SQLException {
		Set<String> subjectCodes=readSubject();
		if(CollectionUtils.isEmpty(subjectCodes)) {
			return;
		}
		System.out.println("总科目数："+subjectCodes.size());
		int total=0;
		for(String code:subjectCodes) {
			List<PropertyDto> props=getProperty(connect, code);
			if(CollectionUtils.isNotEmpty(props)) {
				total++;
				FileUtil.writeFile(sourceDir+"prop/", code+".json", JSONObject.toJSONString(props));
			}else {
				System.out.println("无文件："+code);
			}
		}
		System.out.println("文件数："+total);
	}
	private static Set<String> readSubject() {
		Set<String> list = new HashSet<>();
		XSSFWorkbook wb = null;
		try {
			wb = new XSSFWorkbook(sourceDir + "subject_info.xlsx");
			XSSFSheet sheet = wb.getSheetAt(0);
			int rows = sheet.getLastRowNum();
			for (int i = 1; i <= rows; i++) {
				XSSFRow row = sheet.getRow(i);
				String code = row.getCell(1).getStringCellValue().trim();
				list.add(code);
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			if (wb != null) {
				try {
					wb.close();
				} catch (IOException e) {
				}
			}
		}
		return list;
	}
	
	private static List<PropertyDto> getProperty(Connection connect, String subjectCode) throws SQLException, IOException {
		List<PropertyDto> as = new ArrayList<>();
		PreparedStatement preState = null;
		ResultSet resultSet = null;
		try {
			String sql = " select k.id,k.parent_id,k.type_name,k.type_seq,k.root_id from wq_knowledge_system k "
					+" where k.root_id in "
					+" (select DISTINCT ss.root_id "
					+" from wq_question_bank_subject t "
					+" left join wq_subject sub on t.subject_id=sub.id"
					+" left join wq_question_bank b on t.question_bank_id=b.id "
					+" left join wq_question_question_bank f on t.question_bank_id=f.question_bank_id "
					+" left join wq_question q on f.question_id=q.id "
					+" left join wq_question_knowledge_system s on f.question_id=s.question_id "
					+" left join wq_knowledge_system ss on s.knowledge_system_id=ss.id "
					+" where sub.subject_code='"+subjectCode+"' and b.rent_id=811 and b.is_deleted=0 "
					+" and q.is_deleted=0 and q.rent_id=811) "
					+" ORDER BY k.id ";
			preState = connect.prepareStatement(sql);

			resultSet = preState.executeQuery();
			Map<Long,PropertyDto> map=new HashMap<>();
			while (resultSet.next()) {
				Long rootId=resultSet.getLong("root_id");
				PropertyDto a = new PropertyDto();
				a.setId(resultSet.getLong("id"));
				a.setParentId(resultSet.getLong("parent_id"));
				a.setName(resultSet.getString("type_name"));
				if(a.getParentId()==0) {
					as.add(a);
				}else {
					PropertyDto parent=map.get(a.getParentId());
					if(parent.getParentId()==0) {
						a.setName(a.getName());
					}else {
						a.setName(parent.getName()+"|"+a.getName());
					}
					PropertyDto root=map.get(rootId);
					root.addSub(a);
				}
				map.put(a.getId(), a);
			}
			return as;
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}
}
